Unit ASMUtils;
{
  Assembler Utils - Various functions & procedures written in assembler.
  This unit may not use ANY form of global data or init-code. It's supposed
  to add no overhead to your program.

  Eddy L O Jansson / DFR Research & Engineering
  (fidonet: 2:206/233  -  email: srm_dfr@hotmail.com)
}

INTERFACE

TYPE
 CountryInfoRec = Record
  cID                   :Byte; { = 01 }
  Size                  :Word;
  CountryID             :Word;
  CodePage              :Word;
  DateFormat            :Word;
  CurrencySymbol        :Array[1..5] of Char;
  ThousandSeparator     :Array[1..2] of Char;
  DecimalSeparator      :Array[1..2] of Char;
  DateSeparator         :Array[1..2] of Char;
  TimeSeparator         :Array[1..2] of Char;
  CurrencySymLoc        :Byte;
  CurrencyDecPlaces     :Byte;
  TimeFormat            :Byte;
  ExtASCIImap           :Pointer;
  ListSeparator         :Array[1..2] of Char;
 { Reserved              :Array[1..10] of Byte;}
 end;

Procedure ReleaseTime;    Inline($CD/$28/$B8/$80/$16/$CD/$2F);
{ Release DOS, Windows, Windows '95, and OS/2 timeslices (7 bytes) }

Function  ReadKey: Char;  Inline($B4/$08/$CD/$21);
{
 Description:

  Substitutes the CRT function with the same name. This one is smaller.

 See Also:

  #FlushTgbBuffer# #Waitkey# #KeyPressed#
 }

Procedure FlushTgbBuffer; Inline($B8/$06/$0C/$BA/$FF/$00/$CD/$21);
{
 Description:

  Empty the keyboard buffer.

 See Also:

  #Readkey# #Waitkey# #KeyPressed#
 }

Function  KeyPressed: Boolean;
{
 Description:

  Returns TRUE if a key was pressed (ie, there's data in the keyboard buffer)

 See Also:

  #Readkey# #FlushTgbBuffer# #Waitkey#
 }


Procedure Waitkey;
{
 Description:

  Flushes the keyboard buffer and then waits for a key, discarding it.

 See Also:

  #Readkey# #FlushTgbBuffer# #KeyPressed#
 }

Function  HexL(const L: LongInt): String;
Function  HexW(const W: Word): String;
Function  HexB(const B: Byte): String;

Function  SwapW(W: Word): Word;

Function  Ptr2Str(const P: Pointer): String;
{
  Written: 96-12-03, Eddy L O Jansson (2:206/233)
  Updated: 97-05-28, a little faster on my P133. Nicer code.

  Description:

   Returns (as a pascal string) the zeroterminated string pointed to
   by P.
}

Function  Exist(FileName :String): Boolean;
{
  Written: n/a, Eddy L O Jansson (2:206/233)

  Description:

   Returns true if FILENAME exists. Filename can be a file, a directory
   or even the volumeID. Exist() uses the DOS-function GetFileAttr().
}

Procedure CreateTempFile(var S: String);
{
  Written: 96-12-03, Eddy L O Jansson (2:206/233)
  Updated: 97-05-29, better filename fixup. Internal PString->ZString conv.

  Description:

   Given a directory path (with trailing backslash) this function (using
   DOS) will create an empty file with a _guaranteed unique_ filename.
   The file created is returned in S. This is good for creating temporary
   files without the risk of running into a filename collision.
}

Function  GetActiveCodePage: Word;
{ Return the active codepage, (437/850 etc..) }

Function  GetSystemCodePage: Word;
{ Return the system codepage; the ROM'ed one }

Procedure GetCountryInfo(var CI{: CountryInfoRec};const Size: Word);
{ Return/fill in a block of country-specific data. }

Function  MinLW(const A,B: Longint): LongInt;
Function  MaxLW(const A,B: LongInt): LongInt;
Function  MinLI(const A,B: LongInt): LongInt;
Function  MaxLI(const A,B: LongInt): LongInt;
Function  MinW(const A,B: Word): Word;
Function  MaxW(const A,B: Word): Word;
Function  MinI(const A,B: Integer): Integer;
Function  MaxI(const A,B: Integer): Integer;

Function BitTest1(var BitField;Bit :Word): Boolean;
{
  Written: 97-05-29, Eddy L O Jansson (2:206/233)
  Updated: n/a

  Description:

   Returns TRUE or FALSE for a bit (zerobased) in a data array.
   Important: The array is treated as an array of BITS (binaries).

  See Also:

   #BitTest2#
}

Function BitTest2(var BitField;Bit :Word): Boolean;
{
  Written: 97-05-29, Eddy L O Jansson (2:206/233)
  Updated: n/a

  Description:

   Returns TRUE or FALSE for a bit (zerobased) in a data array.
   Important: The array is treated as an array of BYTES (octets).

  See Also:

   #BitTest1#
}

Procedure Move32(var Source,Destination;Count :Word);

Function CountSameInRow(var InBuf;const Ofs,MaxCount: Word): Word;
{
 Count the number of consecutive same bytes in "InBuf", offset'ed by "Ofs".

 Example:

 S:='*          *';                   (* ten spaces *)
 CountSameInRow(S,1,Length(S)); = 1   (* the first star is alone *)
 CountSameInRow(S,2,Length(S)); = 10  (* the ten spaces *)
 CountSameInRow(S,2,5); = 5           (* as above, but result limited to five *)

 Note:

  This routine, together with it's sister #CountDiffInRow#, are the
  two basic routines needed to implement Run-Length Encoding (RLE).

 See also: #CountByteInRow# #CountDiffInRow#
}

Function CountDiffInRow(var InBuf;const InBufSize,Ofs,MaxCount: Word): Word;
{
 Count the number of /different/ consecutive bytes in an buffer.
 InBufSize specifies the range of the search-space.
 Ofs specifies an offset into the buffer from which the count will start.
 MaxCount specifies a bailout value.

 See Also: #CountSameInRow# #CountByteInRow#
}

Function CountByteInRow(var InBuf;const Ofs,MaxCount: Word;const CountMe: Byte): Word;
{
 Count the number of consecutive same bytes in "InBuf", offset'ed by "Ofs".
 Similar to #CountSameInRow# except you can specify which byte to count.'
}

IMPLEMENTATION

{Function ReadKey: Char; Assembler;
asm
 mov ah,8
 int $21
end;}


{Procedure FlushTgbBuffer; Assembler;  overhead of farcall will be 6 bytes
asm
 mov ax,$0C06;
 mov dx,$00FF;
 int $21;
end;}

Function KeyPressed: Boolean; Assembler;
asm
 mov ax,$40;
 mov es,ax
 mov bx,word ptr es:[$1A]
 cmp bx,word ptr es:[$1C]
 jne @exit
 xor ax,ax
@exit:
end;

Procedure Waitkey; Assembler;
asm
 { First we clear the buffer and wait for a key }
 mov ax,$0C06
 mov dx,$00FF
 int $21
 { then we pop the character entered }
 mov al,8
 int $21
end;

Function HexL(const L: LongInt): String; Assembler;
asm
 les  di,@result
 mov  al,8
 stosb
 jmp  @process
@outALhex:
 mov  ah,al
 shr  al,4
 cmp  al,9
 jng  @first
 add  al,7
@first:
 and  ah,$0F
 cmp  ah,9
 jng  @second
 add  ah,7
@second:
 add  ax,$3030
 stosw
 retf
@process:
 mov  ax,word ptr L+2
 mov  bx,ax
 shr  ax,8
 call far [@outALhex]
 mov  ax,bx
 call far [@outALhex]
 mov  ax,word ptr L
 mov  bx,ax
 shr  ax,8
 call far [@outALhex]
 mov  ax,bx
 call far [@outALhex]
end;

Function HexW(const W: Word): String; Assembler;
asm
 les  di,@result
 mov  al,4
 stosb
 jmp  @process
@outALhex:
 mov  ah,al
 shr  al,4
 cmp  al,9
 jng  @first
 add  al,7
@first:
 and  ah,$0F
 cmp  ah,9
 jng  @second
 add  ah,7
@second:
 add  ax,$3030
 stosw
 retf
@process:
 mov  ax,W
 mov  bx,ax
 shr  ax,8
 call far [@outALhex]
 mov  ax,bx
 call far [@outALhex]
end;

Function HexB(const B: Byte): String; Assembler;
asm
 les  di,@result
 mov  al,2
 stosb
 mov  al,B
 mov  ah,al
 shr  al,4
 cmp  al,9
 jng  @first
 add  al,7
@first:
 and  ah,$0F
 cmp  ah,9
 jng  @second
 add  ah,7
@second:
 add  ax,$3030
 stosw
end; { HexB }

Function SwapW(W: Word): Word; Assembler;
asm
 mov ax,word ptr W
 rol ax,8
end;

Function Ptr2Str(const P: Pointer): String; Assembler;
asm
 push ds
 les  di,P
 xor  ax,ax
 mov  cx,000FFh
 repne scasb
 not  cl
 les  di,@result
 lds  si,P
 mov  al,cl
 dec  al
 stosb
 rep  movsb
 pop  ds
end;

Function Exist(FileName :String): Boolean; Assembler;
asm
 push ds
 lds  si,filename
 xor  bx,bx
 mov  bl,[si]
 inc  si
 mov  dx,si
 mov  [si+bx],bh
 mov  ax,4300h
 int  21h
 mov  al,bh
 adc  cl,cl
 and  cl,31h
 jnz  @exit
 inc  al
@exit:
 pop  ds
end; {FileExists}

Procedure CreateTempFile(var S: String); Assembler;
asm
 push ds
 lds  dx,S
 mov  bx,dx
 xor  ah,ah
 mov  al,byte ptr [bx]
 add  bx,ax
 mov  byte ptr [bx],0  { Add zero-termination }
 inc  dx
 mov  cx,$20
 mov  ah,$5A
 int  $21              { create temp-file }
 jc   @error
 mov  bx,ax
 mov  ah,$3E
 int  $21              { close file }
 jc   @error
 { scan for zero, create pascalstring }
 les  di,S
 xor  ax,ax
 mov  bx,di
 mov  cx,000FFh
 repne scasb
 not  cx
 dec  cx
 dec  cx
 mov  byte ptr [bx],cl
 jmp  @exit
@error:
 mov  bx,dx
 dec  bx
 mov  byte ptr [bx],0 { if error, return empty string }
@exit:
 pop  ds
end; { CreateTempFile }

Function GetActiveCodePage: Word; Assembler;
asm
 mov  ax,$6601
 int  $21
 mov  ax,bx
end;

Function GetSystemCodePage: Word; Assembler;
asm
 mov  ax,$6601
 int  $21
 mov  ax,dx
end;

Procedure GetCountryInfo(var CI{: CountryInfoRec};const Size: Word); Assembler;
asm
 push ds
 mov  ah,$65
 mov  al,01
 xor  bx,bx
 xor  dx,dx
 dec  bx
 dec  dx
 mov  cx,Size { # of bytes to return ($1F with reserved bytes excluded }
 les  di,CI
 int  $21
 pop  ds
end;

Function MinLW(const A,B: Longint): LongInt; Assembler;
asm
 les  ax,A
 mov  dx,es
 cmp  dx,word ptr [B+2]
 jb   @exit
 ja   @pickb
 cmp  ax,word ptr [B+0]
 jbe  @exit
@pickb:
 les  ax,B
 mov  dx,es
@exit:
end;

Function MaxLW(const A,B: LongInt): LongInt; Assembler;
asm
 les  ax,A
 mov  dx,es
 cmp  dx,word ptr [B+2]
 ja   @exit
 jb   @pickb
 cmp  ax,word ptr [B+0]
 jae  @exit
@pickb:
 les  ax,b
 mov  dx,es
@exit:
end;

Function MinLI(const A,B: LongInt): LongInt; Assembler;
asm
 les  ax,A
 mov  dx,es
 cmp  dx,word ptr [B+2]
 jl   @exit
 jg   @pickb
 cmp  ax,word ptr [B+0]
 jbe  @exit
@pickb:
 les  ax,B
 mov  dx,es
@exit:
end;

Function MaxLI(const A,B: LongInt): LongInt; Assembler;
asm
 les  ax,A
 mov  dx,es
 cmp  dx,word ptr [B+2]
 jg   @exit
 jl   @pickb
 cmp  ax,word ptr [B+0]
 jae  @exit
@pickb:
 les  ax,B
 mov  dx,es
@exit:
end;

Function MinW(const A,B: Word): Word; Assembler;
asm
 mov  ax,A
 mov  dx,B
 cmp  ax,dx
 jbe  @exit
 mov  ax,dx
@exit:
end;

Function MaxW(const A,B: Word): Word; Assembler;
asm
 mov  ax,A
 mov  dx,B
 cmp  ax,dx
 jae  @exit
 mov  ax,dx
@exit:
end;

Function MinI(const A,B: Integer): Integer; Assembler;
asm
 mov  ax,A
 mov  dx,B
 cmp  ax,dx
 jle  @exit
 mov  ax,dx
@exit:
end;

Function MaxI(const A,B: Integer): Integer; Assembler;
asm
 mov  ax,A
 mov  dx,B
 cmp  ax,dx
 jge  @exit
 mov  ax,dx
@exit:
end;

Function BitTest1(var BitField;Bit :Word): Boolean; Assembler;
asm
 les di,BitField
 xor dx,dx
 mov ax,Bit
 mov cx,8
 div cx
 add di,ax
 mov al,byte ptr es:[di]
 mov cx,dx
 shr al,cl
 and al,1
end;

Function BitTest2(var BitField;Bit :Word): Boolean; Assembler;
asm
 les di,BitField
 xor dx,dx
 mov ax,Bit
 mov cx,8
 div cx
 add di,ax
 mov al,byte ptr es:[di]
 mov cx,dx
 inc cl
 rol al,cl
 and al,1
end;

Procedure Move32(var Source,Destination ; Count :Word); Assembler;
asm
 mov  dx,ds
 lds  si,source
 les  di,destination
 mov  cx,count
 shr  cx,1
 jnc  @test1
 movsb
@test1:
 shr  cx,1
 jnc  @test2
 movsw
@test2:
 db 66h; rep movsw
 mov ds,dx
end;

Function CountByteInRow(var InBuf;const Ofs,MaxCount: Word;const CountMe: Byte): Word; Assembler;
asm
 cld
 les di,InBuf
 add di,Ofs
 mov cx,MaxCount
 mov dx,cx
 mov al,CountMe
 rep scasb
 jz  @allsame
 dec dx
 sub dx,cx
@allsame:
 mov ax,dx
end;

Function CountSameInRow(var InBuf;const Ofs,MaxCount: Word): Word; Assembler;
asm
 cld
 les di,InBuf
 add di,Ofs
 mov cx,MaxCount
 mov dx,cx
 mov al,byte ptr es:[di]
 inc di
 rep scasb
 sub dx,cx
 mov ax,dx
end;

Function CountDiffInRow(var InBuf;const InBufSize,Ofs,MaxCount: Word): Word; Assembler;
asm
 cld
 les  di,InBuf
 mov  dx,di
 add  dx,InBufSize         { dx = end of buffer }
 add  di,Ofs               { move into buffer }
 mov  cx,MaxCount
 xor  ax,ax                { counter }
 mov  bl,byte ptr es:[di]
@nextpair:
 inc  di
 cmp  dx,di                { is at last byte of buffer? }
 je   @done                { if YES then exit }
 mov  bh,byte ptr es:[di]
 cmp  bl,bh                { same as last byte? }
 je   @done                { if YES then exit }
 xchg bl,bh
 inc  ax                   { this was a different byte, increase counter }
 cmp  ax,cx                { reached MAXCOUNT? }
 jne  @nextpair            { if NO then check next byte }
@done:
end;

END.
